﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace UPP.Engine
{

    public static class DictionaryToArray<T>
    {
        public static T[] ConvertFull(Dictionary<int, T> dictionary, T defaultValue)
        {
            int max = dictionary.Keys.Max() + 1;
            T[] arr = Enumerable.Repeat(defaultValue, max).ToArray();
            foreach (int key in dictionary.Keys)
            {
                arr[key] = dictionary[key];
            }
            return arr;
        }

        public static string[] Pack(Dictionary<int, T> dictionary)
        {
            int max = dictionary.Keys.Count * 2;
            string[] arr = new string[max];
            int i = 0;
            foreach (int key in dictionary.Keys)
            {
                arr[i] = key.ToString();
                arr[i + 1] = dictionary[key].ToString();
                i += 2;
            }
            return arr;
        }

        public static Dictionary<int, T> Unpack(string[] data, Func<string, T> converter)
        {
            int max = data.Length / 2;
            Dictionary<int, T> dict = new Dictionary<int, T>();
            for (int i = 0; i < max; i++)
            {
                dict[Convert.ToInt32(data[i * 2])] = converter(data[(i * 2) + 1]);
            }
            return dict;
        }

        public static class BasicSerializer
        {
            public static string Serialize(Dictionary<int, T> dictionary)
            {
                string dataArrStr = "";

                if (dictionary != null)
                {
                    string[] dataArr = new string[dictionary.Keys.Count * 2];
                    int i = 0;
                    foreach (KeyValuePair<int, T> pair in dictionary)
                    {
                        dataArr[i] = pair.Key.ToString();
                        dataArr[i + 1] = pair.Value.ToString();
                        i += 2;
                    }
                    dataArrStr = Serializer.EncapsulateStrings(dataArr);
                }

                return dataArrStr;
            }

            public static Dictionary<int, T> Deserialize(string data, Func<string, T> converter)
            {
                var dataDes = new Dictionary<int, T>();
                if (data != "")
                {
                    string[] dataArr = Serializer.DecapsulateString(data);
                    if (dataArr != null)
                    {
                        for (int i = 0; i < dataArr.Length; i += 2)
                        {
                            int key = System.Convert.ToInt32(dataArr[i]);
                            T val = converter(dataArr[i + 1]);
                            dataDes[key] = val;
                        }
                    }
                }
                return dataDes;
            }
        }

    }

}