﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Anchor
{
    private bool oldTop;
    public bool Top { get { return oldTop; } set { SetTop(value); } }
    private bool oldBottom;
    public bool Bottom { get { return oldBottom; } set { SetBottom(value); } }
    private bool oldLeft;
    public bool Left { get { return oldLeft; } set { SetLeft(value); } }
    private bool oldRight;
    public bool Right { get { return oldRight; } set { SetRight(value); } }

    public delegate void AnchorChangedHandler(bool topOld, bool topNew, bool rightOld, bool rightNew, bool bottomOld, bool bottomNew, bool leftOld, bool leftNew);
    public event AnchorChangedHandler AnchorChanged;

    public Anchor()
    {
        this.oldTop = true;
        this.oldRight = false;
        this.oldBottom = false;
        this.oldLeft = true;
    }

    public Anchor(bool top, bool right, bool bottom, bool left)
    {
        this.oldTop = top;
        this.oldRight = right;
        this.oldBottom = bottom;
        this.oldLeft = left;
    }

    private void ValidateValues()
    {
        if (!this.oldLeft && !this.oldRight)
            this.oldLeft = true;

        if (!this.oldTop && !this.oldBottom)
            this.oldTop = true;
    }

    private void SetTop(bool newVal)
    {
        if (newVal != oldTop)
        {
            bool xTop = oldTop;
            bool xBottom = oldBottom;
            bool xLeft = oldLeft;
            bool xRight = oldRight;

            oldTop = newVal;
            ValidateValues();

            if (AnchorChanged != null)
                AnchorChanged(xTop, oldTop, xRight, oldRight, xBottom, oldBottom, xLeft, oldLeft);
        }
    }

    private void SetBottom(bool newVal)
    {
        if (newVal != oldBottom)
        {
            bool xTop = oldTop;
            bool xBottom = oldBottom;
            bool xLeft = oldLeft;
            bool xRight = oldRight;

            oldBottom = newVal;
            ValidateValues();

            if (AnchorChanged != null)
                AnchorChanged(xTop, oldTop, xRight, oldRight, xBottom, oldBottom, xLeft, oldLeft);
        }
    }

    private void SetLeft(bool newVal)
    {
        if (newVal != oldLeft)
        {
            bool xTop = oldTop;
            bool xBottom = oldBottom;
            bool xLeft = oldLeft;
            bool xRight = oldRight;

            oldLeft = newVal;
            ValidateValues();

            if (AnchorChanged != null)
                AnchorChanged(xTop, oldTop, xRight, oldRight, xBottom, oldBottom, xLeft, oldLeft);
        }
    }

    private void SetRight(bool newVal)
    {
        if (newVal != oldRight)
        {
            bool xTop = oldTop;
            bool xBottom = oldBottom;
            bool xLeft = oldLeft;
            bool xRight = oldRight;

            oldRight = newVal;
            ValidateValues();

            if (AnchorChanged != null)
                AnchorChanged(xTop, oldTop, xRight, oldRight, xBottom, oldBottom, xLeft, oldLeft);
        }
    }
}
