﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace UPP.OGUI
{

    public class CheckBox : Inline
    {
        private string _text = "";
        public string Text { get { return _text; } set { ChangeText(value); } }

        bool _value = false;
        public bool Checked { get { return _value; } set { SetValue(value); } }

        public delegate void OnChangeHandler(object sender);
        public event OnChangeHandler OnChange;

        private void SetValue(bool newValue)
        {
            if (newValue == _value)
                return;

            _value = newValue;
            if (OnChange != null)
                OnChange(this);
        }

        private void ChangeText(string newVal)
        {
            if (newVal != _text)
            {
                _text = newVal;
                if (AutoSize)
                    SetSizeFromContent();
            }
        }

        public override GUIContent GetContent()
        {
            return new GUIContent(_text);
        }

        public override GUIStyle DefaultStyle()
        {
            GUIStyle skin = new GUIStyle(GUI.skin.toggle);
            skin.margin = new RectOffset(3, 0, 1, 0);
            skin.padding = new RectOffset(18, 18, 1, 1);
            return new GUIStyle(skin);
        }

        private void DrawInherited()
        {
            if (AutoSize)
            {
                bool v = GUILayout.Toggle(_value, _text, GetCurrentStyle());
                if (v != _value)
                    this.Checked = v;
            }
            else
            {
                bool v = GUILayout.Toggle(_value, _text, GetCurrentStyle(),
                    GUILayout.Width(this.Size.Width - GetCurrentStyle().padding.left - GetCurrentStyle().padding.right),
                    GUILayout.Height(this.Size.Height - GetCurrentStyle().padding.top - GetCurrentStyle().padding.bottom)
                );
                if (v != _value)
                    this.Checked = v;
            }
        }

        private void Init(bool check, string text)
        {
            _text = text;
            _value = check;
            this.InheritedContent = DrawInherited;
            SetSizeFromContent();
            this.AutoSize = false;
            this.Size = new UPP.Size(150, GetCurrentStyle().CalcHeight(new GUIContent(), 150)
                + GetCurrentStyle().padding.top + GetCurrentStyle().padding.bottom);
        }

        public CheckBox()
        {
            Init(false, "CheckBox");
        }

        public CheckBox(bool check=false, string text="CheckBox")
        {
            Init(check, text);
        }
        public CheckBox(string text = "CheckBox", bool check = false)
        {
            Init(check, text);
        }


    }

}
