﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UPP.OGUI;
using UnityEditor;
using System;

namespace UPP.OGUI
{

    public class Label : Inline
    {

        private string _text = "";
        public string Text { get { return _text; } set { ChangeText(value); } }

        private Font _font = null;
        public Font Font { get { return _font; } set { SetFont(value); } }

        private GUIStyle _style = null;

        private void SetFont(Font newFont)
        {
            if (newFont == null)
                newFont = GetCurrentStyle().font;
            _font = newFont;
            _style.font = newFont;
        }

        private void Init(string text, Font font, bool bold, bool italic, int size)
        {
            _style = GetCurrentStyle();
            _text = text;
            SetFont(font);

            if (size > 0)
                _style.fontSize = size;
            else
                _style.fontSize = GetCurrentStyle().fontSize;

            if (bold && italic)
                _style.fontStyle = FontStyle.BoldAndItalic;
            if (bold && !italic)
                _style.fontStyle = FontStyle.Bold;
            if (!bold && italic)
                _style.fontStyle = FontStyle.Italic;
            if (!bold && !italic)
                _style.fontStyle = FontStyle.Normal;

            this.InheritedContent = DrawInherited;
            SetSizeFromContent();
        }

        public Label()
        {
            Init(this.Name, null, false, false, -1);
        }

        public Label(string text)
        {
            Init(text, null, false, false, -1);
        }

        public Label(string text, Font font)
        {
            Init(text, font, false, false, -1);
        }

        public Label(string text, bool bold)
        {
            Init(text, null, bold, false, -1);
        }

        public Label(string text, bool bold, bool italic)
        {
            Init(text, null, bold, italic, -1);
        }

        public Label(string text, int size)
        {
            Init(text, null, false, false, size);
        }

        public Label(string text, bool bold, bool italic, int size)
        {
            Init(text, null, bold, italic, size);
        }

        private void DrawInherited()
        {
            if (AutoSize)
                GUILayout.Label(_text, GetCurrentStyle());
            else
            {
                GUILayout.Label(_text, GetCurrentStyle(),
                    GUILayout.Width(this.Size.Width - GetCurrentStyle().padding.left - GetCurrentStyle().padding.right),
                    GUILayout.Height(this.Size.Height - GetCurrentStyle().padding.top - GetCurrentStyle().padding.bottom)
                );
            }
        }

        private void ChangeText(string newVal)
        {
            if (newVal != _text)
            {
                _text = newVal;
                if (AutoSize)
                    SetSizeFromContent();
            }
        }

        public override GUIContent GetContent()
        {
            return new GUIContent(this.Text);
        }

        public override GUIStyle DefaultStyle()
        {
            GUIStyle skin = new GUIStyle(GUI.skin.label);
            skin.margin = new RectOffset(0, 0, 0, 0);
            return new GUIStyle(skin);
        }
    }

}