﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace UPP.OGUI
{
    public class TableRow
    {
        private float _height = 20;
        public float Height { get { return _height; } set { SetHeight(value); } }

        private GUIStyle _style = new GUIStyle();
        public GUIStyle Style { get { return _style; } set { SetStyle(value); } }

        public List<string> Cells;

        private int _expectedColCount = 0;
        public int ExpectedColCount { get { return _expectedColCount; } set { SetColCount(value); } }

        private bool _isTitle = false;
        public bool IsTitle { get { return _isTitle; } set { _isTitle = value; } }

        public int rowId = -1;

        public string Tag { get; set; }

        private void SetColCount(int colCnt)
        {
            _expectedColCount = colCnt;
            if (Cells.Count > colCnt)
                for (int i = colCnt; i < Cells.Count; i++)
                    Cells.RemoveAt(i);
        }

        private void SetHeight (float newHeight)
        {
            _height = newHeight;
            _style.fixedHeight = _height;
        }

        private void SetStyle (GUIStyle newStyle)
        {
            _style = newStyle;
            _style.fixedHeight = _height;
        }

        private void Init()
        {
            Cells = new List<string>();
        }

        public TableRow() { Init(); }

        public TableRow(GUIStyle style)
        {
            Init();
            SetStyle(style);
        }

        public void AddValue(string value)
        {
            Cells.Add(value);

        }

        public void SetValue(int col, string value)
        {
            if (Cells.Count <= col)
            {
                int missing = col - Cells.Count;
                for (int i = 0; i < missing; i++)
                    AddValue("");
                AddValue(value);
            }
            else
            {
                Cells[col] = value;
            }
        }

    }
}