﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;

namespace UPP {

    public static class Settings {

        public static readonly string SETTINGS_DB_PATH = "Assets/Unity++/Settings/Data/Database.asset";

        private static SettingsDatabase _settings = null;

        private static void CreateDatabase()
        {
            _settings = ScriptableObject.CreateInstance<SettingsDatabase>();
            AssetDatabase.CreateAsset(_settings, SETTINGS_DB_PATH);

            AssetDatabase.Refresh();
            EditorUtility.SetDirty(_settings);
            AssetDatabase.SaveAssets();

            _settings.database = new List<SettingsRecord>();
        }

        public static List<SettingsRecord> GetAll()
        {
            if (_settings == null)
                LoadDatabase();

            if (_settings == null)
            {
                Debug.LogError("There was an error creating Unity++ settings database.");
                return null;
            }

            return _settings.database;
        }

        private static void LoadDatabase()
        {
            _settings = (SettingsDatabase)AssetDatabase.LoadAssetAtPath(SETTINGS_DB_PATH, typeof(SettingsDatabase));
            if (_settings == null)
                CreateDatabase();
        }

        public static string Get(string key)
        {
            return Get(key, null);
        }

        public static string GetOrSet(string key, string defValue)
        {
            if (_settings == null)
                LoadDatabase();

            if (_settings == null)
            {
                Debug.LogError("There was an error creating Unity++ settings database.");
                return null;
            }

            SettingsRecord rec = _settings.database.Find(x => x.key == key);

            if (rec == null)
            {
                Set(key, defValue);
                return defValue;
            }

            return rec.value;
        }

        public static string Get(string key, string valueIfNotFound)
        {
            if (_settings == null)
                LoadDatabase();

            if (_settings == null)
            {
                Debug.LogError("There was an error creating Unity++ settings database.");
                return null;
            }

            SettingsRecord rec = _settings.database.Find(x => x.key == key);

            if (rec == null) 
                return valueIfNotFound;

            return rec.value;
        }

        public static void Set(string key, string value)
        {
            if (_settings == null)
                LoadDatabase();

            if (_settings == null)
            {
                Debug.LogError("There was an error creating Unity++ settings database.");
                return;
            }

            SettingsRecord rec = _settings.database.Find(x => x.key == key);

            if (rec != null)
            {
                if (value == null)
                    _settings.database.Remove(rec);
                else
                    rec.value = value;
            }
            else
            {
                if (value != null)
                    _settings.database.Add(new SettingsRecord(key, value));
            }

            AssetDatabase.Refresh();
            EditorUtility.SetDirty(_settings);
            AssetDatabase.SaveAssets();

        }
    }
}