﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UPP.OGUI;

namespace UPP.Engine
{
    public class PropertyWindowInstanceSettings : OGUI.Window
    {
        private const int width = 340;
        private const int height = 380;

        public PropertyRecord PropertyRecord = null;
        private PropertyBase deserializedProp = null;

        OGUI.TextBox tbName;
        OGUI.TextBox tbDesc;
        OGUI.TextBox tbMachine;
        OGUI.TextBox tbCategory;

        CheckBox chbTemplate;
        CheckBox chbInstance;

        NumBox tbCount;
        CheckBox chbCountInf;

        ComboBox cbPropag;

        private void fillPropag(string key)
        {
            cbPropag.Clear();
            foreach (PropertyDBRecord rec in PropertyManager.DB.database)
            {
                if (rec.machineClassName == PropertyRecord.ClassMachineName &&
                    rec.machinePropertyName != deserializedProp.MachineName)
                {
                    var o = PropertyManager.GetPropertyByMachineName(rec.machineClassName, rec.machinePropertyName);

                    if (o.PropertyType == PropertyRecord.PropertyType)
                    {
                        var obj = PropertyBase.FromSerialized(o.PropertyType, o.PropertySerialized);
                        cbPropag.Add(obj.MachineName, obj.Name);
                    }
                }
            }
            cbPropag.SelectedKey = key;
        }

        protected override void PrepareGUI()
        {
            if (PropertyRecord == null)
            {
                Debug.LogError("PropertyRecord not set while opening PropertyWindowInstanceSettings.");
                this.Close();
                return;
            }

            position = new Rect(position.x, position.y, width, height);
            minSize = new Vector2(width, height);

            var pr = PropertyBase.FromSerialized(PropertyRecord.PropertyType, PropertyRecord.PropertySerialized);
            deserializedProp = pr;

            PropertyInfo prInfo = (PropertyInfo)Attribute.GetCustomAttribute(pr.GetType(), typeof(PropertyInfo));

            OGUI.Label l2 = new OGUI.Label("[" + prInfo.Name + "]", true);
            l2.Anchor = new Anchor(true, true, false, false);
            l2.Position = new Position(width - l2.Size.Width - 10, 10);

            OGUI.Label l1 = new OGUI.Label("Readable name:");
            l1.Position = new Position(10, 35);

            tbName = new OGUI.TextBox(pr.Name, 31);
            tbName.Anchor = new Anchor(true, true, false, true);
            tbName.Position = new Position(160, 35);
            tbName.Size.Width = 176;

            OGUI.Label l5 = new OGUI.Label("Machine name:");
            l5.Position = new Position(10, 60);

            tbMachine = new OGUI.TextBox(pr.MachineName, 31);
            tbMachine.Anchor = new Anchor(true, true, false, true);
            tbMachine.Position = new Position(160, 60);
            tbMachine.Size.Width = 176;
            tbMachine.Mask = MaskTemplates.MachineName;

            OGUI.Label l3 = new OGUI.Label("Description:");
            l3.Position = new Position(10, 85);

            tbDesc = new OGUI.TextBox(pr.Description, 255);
            tbDesc.Anchor = new Anchor(true, true, false, true);
            tbDesc.Position = new Position(160, 85);
            tbDesc.Size.Width = 176;

            Label l4 = new Label("Category:");
            l4.Position = new Position(10, 110);

            tbCategory = new OGUI.TextBox(pr.Category, 31);
            tbCategory.Anchor = new Anchor(true, true, false, true);
            tbCategory.Position = new Position(160, 110);
            tbCategory.Size.Width = 176;

            OGUI.Label l6 = new OGUI.Label("Expose to Templates:");
            l6.Position = new Position(10, 135);
            OGUI.Label l7 = new OGUI.Label("Expose to Instances:");
            l7.Position = new Position(10, 160);
            OGUI.Label l8 = new OGUI.Label("Value count:");
            l8.Position = new Position(10, 185);
            OGUI.Label l10 = new OGUI.Label("Value propagation:");
            l10.Position = new Position(10, 210);

            chbTemplate = new CheckBox("Yes", pr.ExposeToTemplates);
            chbTemplate.Position = new Position(160, 135);
            chbInstance = new CheckBox("Yes", pr.ExposeToInstances);
            chbInstance.Position = new Position(160, 160);
            tbCount = new NumBox(pr.ValueCount, 1, 10000);
            tbCount.Position = new Position(160, 185);
            tbCount.Size.Width = 100;
            tbCount.Enabled = pr.ValueCount != 0;
            chbCountInf = new CheckBox("Unlimited", pr.ValueCount == 0);
            chbCountInf.Position = new Position(255, 185);
            chbCountInf.OnChange += ChbCountInf_OnChange;

            cbPropag = new ComboBox("<Do not inherit>");
            cbPropag.Anchor = new Anchor(true, true, false, true);
            cbPropag.Position = new Position(160, 210);
            cbPropag.Size.Width = 176;
            fillPropag(pr.ValuePropagation);

            OGUI.Label l9 = new OGUI.Label("Settings:");
            l9.Position = new Position(10, 235);

            OGUI.DialogConfirmPanel p = new OGUI.DialogConfirmPanel(this, position, "Save", "Cancel");
            p.OnConfirm += P_OnConfirm;

            OGUI.Panel set = new OGUI.Panel();
            set.Anchor = new Anchor(true, true, true, true);
            set.Size = new Size(width - 20, height - 295);
            set.Position = new Position(10, 260);
            set.SizeMode = OGUI.Panel.SizeModeType.ScrollBars;
            set.BackgroundColor = ColorSchemes.BackgroundColorDarker;
            set.BorderColor = ColorSchemes.BorderColorDarker;

            pr.GUIForSetting(set);
        }

        private void ChbCountInf_OnChange(object sender)
        {
            tbCount.Enabled = !(chbCountInf.Checked);
        }

        private void P_OnConfirm(DialogConfirmPanel.ConfirmFeedback f)
        {
            // On Save dialog.
            var pr = deserializedProp;

            pr.Name = tbName.Value;
            pr.MachineName = tbMachine.Value;
            pr.Description = tbDesc.Value;
            pr.Category = tbCategory.Value.Trim();
            pr.ExposeToTemplates = chbTemplate.Checked;
            pr.ExposeToInstances = chbInstance.Checked;
            if (chbCountInf.Checked)
                pr.ValueCount = 0;
            else
                pr.ValueCount = tbCount.Value;
            if (cbPropag.SelectedIndex != -1)
                pr.ValuePropagation = cbPropag.SelectedKey;
            else
                pr.ValuePropagation = "";

            pr.SaveGUIForSetting();

            PropertyRecord.PropertySerialized = pr.ToSerialized(PropertyRecord.PropertyType);
        }
    }

}