﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UPP.OGUI;

namespace UPP.Engine
{

    public class PropertyWindowInstanceValues : Window
    {
        private int width = 340;
        private int height = 150;

        public PropertyBase DeserializedProp = null;

        int Page = 0;
        NumBox PageBox;

        protected override void PrepareGUI()
        {
            if (DeserializedProp == null)
            {
                Debug.LogError("PropertyRecord not set while opening PropertyWindowInstanceValues.");
                this.Close();
                return;
            }

            var pr = DeserializedProp;

            if (pr.ValueCount == 1)
            {
                height -= 35;
            }

            position = new Rect(position.x, position.y, width, height);
            minSize = new Vector2(width, height);

            int yoff = 0;

            NumBox nbox = null;

            if (pr.ValueCount != 1)
            {
                yoff = 35;     
                Panel p = new Panel();
                p.Position = new Position(10, 10);
                p.Size = new Size(width - 20, 25);
                p.Anchor = new Anchor(true, true, false, true);

                int max = pr.ValueCount;
                if (max == 0)
                    max = Int32.MaxValue;
                nbox = new NumBox(0, 0, max);
                nbox.Size.Width = 60;
                nbox.Anchor = new Anchor(true, false, false, false);
                nbox.Parent = p;
                float w = nbox.Size.Width;
                if (pr.ValueCount != 0)
                {
                    Label lm = new Label("/ " + (pr.ValueCount - 1).ToString());
                    w += lm.Size.Width;
                    nbox.Position = new Position((p.Size.Width / 2) - (w / 2), 4);
                    lm.Position = new Position(nbox.Position.X + nbox.Size.Width, 4);
                    lm.Parent = p;
                    lm.Anchor = new Anchor(true, false, false, false);
                }
                else
                {
                    nbox.Position = new Position((p.Size.Width / 2) - (w / 2), 4);
                }

                Button bLeft = new Button("<<");
                int bl = nbox.GetCurrentStyle().margin.left + nbox.GetCurrentStyle().padding.left + 
                    bLeft.GetCurrentStyle().margin.left + bLeft.GetCurrentStyle().padding.left;
                bLeft.Parent = p;
                bLeft.Anchor = new Anchor(true, false, false, false);
                bLeft.AutoSize = false;
                bLeft.Size.Width = 50;
                bLeft.Position = new Position((nbox.Position.X - bLeft.Size.Width) + bl, 4);
                bLeft.OnClick += BLeft_OnClick;

                Button bRight = new Button(">>");
                int br = 0;
                if (pr.ValueCount == 0)
                    br = nbox.GetCurrentStyle().margin.right + nbox.GetCurrentStyle().padding.right;
                bRight.Parent = p;
                bRight.Anchor = new Anchor(true, false, false, false);
                bRight.AutoSize = false;
                bRight.Size.Width = 50;
                bRight.Position = new Position(nbox.Position.X + w - br, 4);
                bRight.OnClick += BRight_OnClick;

                nbox.BringToFront();
                bRight.BringToFront();
            }

            PageBox = nbox;

            new OGUI.DialogConfirmPanel(this, position, "Done", null);

            OGUI.Panel set = new OGUI.Panel();
            set.Anchor = new Anchor(true, true, true, true);

            set.Size = new Size(width - 20, height - 45 - yoff);
            set.Position = new Position(10, 10 + yoff);
            set.SizeMode = OGUI.Panel.SizeModeType.ScrollBars;
            set.BackgroundColor = ColorSchemes.BackgroundColorDarker;
            set.BorderColor = ColorSchemes.BorderColorDarker;

            pr.GUIForValue(set);

            if (nbox != null)
                nbox.OnChange += Nbox_OnChange;
            
        }

        private void BRight_OnClick(object sender)
        {
            GUIUtility.keyboardControl = 0;
            SetPage(Page + 1);
        }

        private void BLeft_OnClick(object sender)
        {
            GUIUtility.keyboardControl = 0;
            SetPage(Page - 1);
        }

        private void Nbox_OnChange(object sender)
        {
             SetPage((sender as NumBox).Value);
        }

        private void SetPage(int index)
        {

            if (index < 0)
                index = 0;
            if (index >= DeserializedProp.ValueCount && DeserializedProp.ValueCount != 0)
                index = DeserializedProp.ValueCount - 1;

            Page = index;

            if (PageBox != null && PageBox.Value != Page)
                PageBox.Value = Page;

            DeserializedProp.GUIForValueSetPage(Page);
        }
    }

}